
/* -----------------------------------------------------------------------
 * This is TIFF support to pdfTeX.
 * -----------------------------------------------------------------------
 */

#include "libpdftex.h"
#include "image.h"

/* Function tiff_read_image_info opens specified file and reads informations
 * into TIFF_IMAGE_INFO structure.
 *
 * Return value: 0 - OK
 *               1 - Cannot open file for read or not a TIFF file
 */

integer read_tiff_info(integer img)
{
  TIFF_IMAGE_INFO *image_info = TIFF_INFO(img);

  image_info->file = (FILE *) TIFFOpen(IMG_NAME(img), FOPEN_RBIN_MODE);

  if (!image_info->file)
    return 1; /* This is not a TIFF file */

  image_info->x_res=image_info->y_res=0;

  TIFFGetField((TIFF *) image_info->file, TIFFTAG_IMAGELENGTH, &image_info->height);
  TIFFGetField((TIFF *) image_info->file, TIFFTAG_IMAGEWIDTH, &image_info->width);

  image_info->length=3*image_info->height*image_info->width;
  image_info->color_space=0;
  image_info->bits_per_component=8;

return 0;
}

void write_tiff(integer n, integer img)
{

    long unsigned row, col;
    unsigned long int *raster=(unsigned long int *) _TIFFmalloc(TIFF_INFO(img)->height*TIFF_INFO(img)->width*sizeof(unsigned long int));

    pdfbegindict(n);
    PDF_PRINTF("/Type /XObject\n/Subtype /Image\n/Width %li\n/Height %li\n/BitsPerComponent %li\n" AND
               (long int)TIFF_INFO(img)->width AND
               (long int)TIFF_INFO(img)->height AND
               (long int)TIFF_INFO(img)->bits_per_component);
    PDF_PRINTF("/ColorSpace /DeviceRGB\n");

    TIFFReadRGBAImage((TIFF *) TIFF_INFO(img)->file, TIFF_INFO(img)->width, TIFF_INFO(img)->height, raster,0);

    pdfbeginstream();

    for (row = 0; row<TIFF_INFO(img)->height; row++)
       for (col = 0; col<TIFF_INFO(img)->width; col++)
	{
	pdfbuf[pdfptr++] = TIFFGetR(*(raster+col+TIFF_INFO(img)->width*(TIFF_INFO(img)->height-1-row)));
	if (pdfptr == pdfbufsize)
		pdfflush();
	pdfbuf[pdfptr++] = TIFFGetG(*(raster+col+TIFF_INFO(img)->width*(TIFF_INFO(img)->height-1-row)));
	if (pdfptr == pdfbufsize)
		pdfflush();
	pdfbuf[pdfptr++] = TIFFGetB(*(raster+col+TIFF_INFO(img)->width*(TIFF_INFO(img)->height-1-row)));
	if (pdfptr == pdfbufsize)
		pdfflush();
	}
    pdfendstream();
}
